@@ -45,7 +45,7 @@
    return NULL if there are no more elements.  */
 
 char *
-next_element (char *new_path, int curdir_ok)
+next_element (const char *new_path, int curdir_ok)
 {
   static char *path = NULL;	/* Freshly allocated copy of NEW_PATH.  */
   static char *end;		/* Start of next element to return.  */
@@ -66,7 +66,7 @@ next_element (char *new_path, int curdir_ok)
       if (final_colon)
 	{
 	  final_colon = 0;
-	  return curdir_ok ? "." : NULL;
+	  return curdir_ok ? "." : "";
 	}
       return NULL;
     }
@@ -79,7 +79,7 @@ next_element (char *new_path, int curdir_ok)
     {
       /* An empty path element.  */
       *end++ = '\0';
-      return curdir_ok ? "." : NULL;
+      return curdir_ok ? "." : "";
     }
   else if (end == NULL)
     {
